require 'json'
require 'net/http'
require 'net/http/post/multipart'

require "glossyapp/session/documents"
require "glossyapp/session/publications"
require "glossyapp/session/pages"
require "glossyapp/session/page_elements"
require "glossyapp/session/assets"

# Fix broken patron URL builder
require "ext/session"


module GlossyApp
  
  # You find the rest of the methods in glossyapp/session/*
  class Session
    
    def initialize(api_key, base_url = 'http://api.glossyapp.com/v1', proxy = nil, timeout = 30)
      
      @api_key = api_key
      @base_url = base_url
      @proxy = proxy
      @timeout = timeout
      
      @user_agent = "glossyapp-api-ruby/#{GlossyApp::VERSION}"
      
      # Handle Proxy
      if @proxy
        raise "Setting a proxy is no longer supported"
      end
    end
    
    def ping
      response = glossy_get "/ping"
      if [200].include?(response.status)
        return true
      else
        return false
      end
    end
    
    
    protected
    
    def glossy_get(path, headers = {})
      request = Net::HTTP::Get.new(build_uri(path))
      return make_request(request, headers)
    end
    
    def glossy_put(path, data, headers = {})
      request = Net::HTTP::Put.new(build_uri(path))
      request.body = data
      return make_request(request, headers)
    end
    
    def glossy_post(path, data, headers = {})
      request = Net::HTTP::Post.new(build_uri(path))
      request.body = data
      return make_request(request, headers)
    end
    
    def glossy_post_multipart(path, data = {}, files = {}, headers = {})
      raise ArgumentError, "data needs to be a hash" unless data.is_a? Hash
      raise ArgumentError, "files needs to be a hash" unless files.is_a? Hash
      
      post_data = {}
      data.each_pair  { |key, value| post_data[key] = value }
      files.each_pair { |key, value| post_data[key] = UploadIO.new(File.new(value), 'application/octet-stream', File.basename(value)) }
      
      uri = build_uri(path)
      request = Net::HTTP::Post::Multipart.new(uri,  post_data)

      return make_request(request, headers)
    end
    
    def glossy_delete(path, headers = {})
      request = Net::HTTP::Delete.new(build_uri(path))
      return make_request(request, headers)
    end
    
    
    
    
    
    def build_uri(path)
      return URI("#{@base_url}#{path}")
    end
    
    def make_request(request, headers)
      request['Accept'] = 'application/json'
      request['User-Agent'] = @user_agent
      
      headers.each_pair { |key, value| request[key] = value }
      
      request.basic_auth @api_key, ''
      
      response = Net::HTTP.start(request.uri.hostname, request.uri.port, :use_ssl => request.uri.scheme == 'https', :read_timeout => @timeout) do |http|
        http.request(request)
      end
      
      return GlossyResponse.new(response.code.to_i, response.body)
    end
    
    
  end
  
  class GlossyResponse
    attr_accessor :status
    attr_accessor :body
    
    def initialize(status, body)
      @status = status
      @body = body
    end
  end
  
end
